@@ -34,6 +34,7 @@ from bokeh.themes import Theme
 from bokeh.transform import factor_cmap, linear_cmap
 from bokeh.io import curdoc
 import dask
+from dask import config
 from dask.utils import format_bytes, key_split
 from tlz import pipe
 from tlz.curried import map, concat, groupby
@@ -74,7 +75,9 @@ logger = logging.getLogger(__name__)
 from jinja2 import Environment, FileSystemLoader
 
 env = Environment(
-    loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), "..", "templates"))
+    loader=FileSystemLoader(
+        os.path.join(os.path.dirname(__file__), "..", "..", "http", "templates")
+    )
 )
 
 BOKEH_THEME = Theme(os.path.join(os.path.dirname(__file__), "..", "theme.yaml"))
@@ -1090,7 +1093,7 @@ def task_stream_figure(clear_interval="20s", **kwargs):
             """,
     )
 
-    tap = TapTool(callback=OpenURL(url="/profile?key=@name"))
+    tap = TapTool(callback=OpenURL(url="./profile?key=@name"))
 
     root.add_tools(
         hover,
@@ -1171,6 +1174,7 @@ class TaskGraph(DashboardComponent):
         tap = TapTool(callback=OpenURL(url="info/task/@key.html"), renderers=[rect])
         rect.nonselection_glyph = None
         self.root.add_tools(hover, tap)
+        self.max_items = config.get("distributed.dashboard.graph-max-items", 5000)
 
     @without_property_validation
     def update(self):
@@ -1206,6 +1210,10 @@ class TaskGraph(DashboardComponent):
             y = self.layout.y
 
             tasks = self.scheduler.tasks
+            if len(tasks) > self.max_items:
+                # graph to big - no update, reset for next time
+                self.invisible_count = len(tasks)
+                return
             for key in new:
                 try:
                     task = tasks[key]
@@ -1659,9 +1667,8 @@ def systemmonitor_doc(scheduler, extra, doc):
         doc.title = "Dask: Scheduler System Monitor"
         add_periodic_callback(doc, sysmon, 500)
 
-        for subdoc in sysmon.root.children:
-            doc.add_root(subdoc)
-        doc.template = env.get_template("system.html")
+        doc.add_root(sysmon.root)
+        doc.template = env.get_template("simple.html")
         doc.template_variables.update(extra)
         doc.theme = BOKEH_THEME
 
