@@ -1,3 +1,4 @@
+import asyncio
 import json
 import re
 import ssl
@@ -7,8 +8,8 @@ from time import sleep
 import pytest
 
 pytest.importorskip("bokeh")
+from bokeh.server.server import BokehTornado
 from tlz import first
-from tornado import gen
 from tornado.httpclient import AsyncHTTPClient, HTTPRequest
 
 import dask
@@ -17,9 +18,8 @@ from distributed.utils import tokey, format_dashboard_link
 from distributed.client import wait
 from distributed.metrics import time
 from distributed.utils_test import gen_cluster, inc, dec, slowinc, div, get_cert
-from distributed.dashboard.worker import BokehWorker
 from distributed.dashboard.components.worker import Counters
-from distributed.dashboard.scheduler import applications, BokehScheduler
+from distributed.dashboard.scheduler import applications
 from distributed.dashboard.components.scheduler import (
     SystemMonitor,
     Occupancy,
@@ -36,41 +36,34 @@ from distributed.dashboard.components.scheduler import (
     ProfileServer,
     MemoryByKey,
 )
-
 from distributed.dashboard import scheduler
 
 scheduler.PROFILING = False
 
 
-@pytest.mark.skipif(
-    sys.version_info[0] == 2, reason="https://github.com/bokeh/bokeh/issues/5494"
-)
-@gen_cluster(
-    client=True, scheduler_kwargs={"services": {("dashboard", 0): BokehScheduler}}
-)
-def test_simple(c, s, a, b):
-    assert isinstance(s.services["dashboard"], BokehScheduler)
-    port = s.services["dashboard"].port
+@gen_cluster(client=True, scheduler_kwargs={"dashboard": True})
+async def test_simple(c, s, a, b):
+    port = s.http_server.port
 
     future = c.submit(sleep, 1)
-    yield gen.sleep(0.1)
+    await asyncio.sleep(0.1)
 
     http_client = AsyncHTTPClient()
     for suffix in applications:
-        response = yield http_client.fetch("http://localhost:%d%s" % (port, suffix))
+        response = await http_client.fetch("http://localhost:%d%s" % (port, suffix))
         body = response.body.decode()
         assert "bokeh" in body.lower()
         assert not re.search("href=./", body)  # no absolute links
 
-    response = yield http_client.fetch(
+    response = await http_client.fetch(
         "http://localhost:%d/individual-plots.json" % port
     )
     response = json.loads(response.body.decode())
     assert response
 
 
-@gen_cluster(client=True, worker_kwargs=dict(services={"dashboard": BokehWorker}))
-def test_basic(c, s, a, b):
+@gen_cluster(client=True, worker_kwargs={"dashboard": True})
+async def test_basic(c, s, a, b):
     for component in [TaskStream, SystemMonitor, Occupancy, StealingTimeSeries]:
         ss = component(s)
 
@@ -82,24 +75,24 @@ def test_basic(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_counters(c, s, a, b):
+async def test_counters(c, s, a, b):
     pytest.importorskip("crick")
     while "tick-duration" not in s.digests:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
     ss = Counters(s)
 
     ss.update()
-    yield gen.sleep(0.1)
+    await asyncio.sleep(0.1)
     ss.update()
 
     start = time()
     while not len(ss.digest_sources["tick-duration"][0].data["x"]):
-        yield gen.sleep(1)
+        await asyncio.sleep(1)
         assert time() < start + 5
 
 
 @gen_cluster(client=True)
-def test_stealing_events(c, s, a, b):
+async def test_stealing_events(c, s, a, b):
     se = StealingEvents(s)
 
     futures = c.map(
@@ -107,7 +100,7 @@ def test_stealing_events(c, s, a, b):
     )
 
     while not b.task_state:  # will steal soon
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     se.update()
 
@@ -115,7 +108,7 @@ def test_stealing_events(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_events(c, s, a, b):
+async def test_events(c, s, a, b):
     e = Events(s, "all")
 
     futures = c.map(
@@ -123,7 +116,7 @@ def test_events(c, s, a, b):
     )
 
     while not b.task_state:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     e.update()
     d = dict(e.source.data)
@@ -131,12 +124,12 @@ def test_events(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_task_stream(c, s, a, b):
+async def test_task_stream(c, s, a, b):
     ts = TaskStream(s)
 
     futures = c.map(slowinc, range(10), delay=0.001)
 
-    yield wait(futures)
+    await wait(futures)
 
     ts.update()
     d = dict(ts.source.data)
@@ -149,7 +142,7 @@ def test_task_stream(c, s, a, b):
     assert all(len(L) == 10 for L in d.values())
 
     total = c.submit(sum, futures)
-    yield wait(total)
+    await wait(total)
 
     ts.update()
     d = dict(ts.source.data)
@@ -157,21 +150,21 @@ def test_task_stream(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_task_stream_n_rectangles(c, s, a, b):
+async def test_task_stream_n_rectangles(c, s, a, b):
     ts = TaskStream(s, n_rectangles=10)
     futures = c.map(slowinc, range(10), delay=0.001)
-    yield wait(futures)
+    await wait(futures)
     ts.update()
 
     assert len(ts.source.data["start"]) == 10
 
 
 @gen_cluster(client=True)
-def test_task_stream_second_plugin(c, s, a, b):
+async def test_task_stream_second_plugin(c, s, a, b):
     ts = TaskStream(s, n_rectangles=10, clear_interval=10)
     ts.update()
     futures = c.map(inc, range(10))
-    yield wait(futures)
+    await wait(futures)
     ts.update()
 
     ts2 = TaskStream(s, n_rectangles=5, clear_interval=10)
@@ -179,21 +172,21 @@ def test_task_stream_second_plugin(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_task_stream_clear_interval(c, s, a, b):
+async def test_task_stream_clear_interval(c, s, a, b):
     ts = TaskStream(s, clear_interval=200)
 
-    yield wait(c.map(inc, range(10)))
+    await wait(c.map(inc, range(10)))
     ts.update()
-    yield gen.sleep(0.010)
-    yield wait(c.map(dec, range(10)))
+    await asyncio.sleep(0.010)
+    await wait(c.map(dec, range(10)))
     ts.update()
 
     assert len(set(map(len, ts.source.data.values()))) == 1
     assert ts.source.data["name"].count("inc") == 10
     assert ts.source.data["name"].count("dec") == 10
 
-    yield gen.sleep(0.300)
-    yield wait(c.map(inc, range(10, 20)))
+    await asyncio.sleep(0.300)
+    await wait(c.map(inc, range(10, 20)))
     ts.update()
 
     assert len(set(map(len, ts.source.data.values()))) == 1
@@ -202,11 +195,11 @@ def test_task_stream_clear_interval(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_TaskProgress(c, s, a, b):
+async def test_TaskProgress(c, s, a, b):
     tp = TaskProgress(s)
 
     futures = c.map(slowinc, range(10), delay=0.001)
-    yield wait(futures)
+    await wait(futures)
 
     tp.update()
     d = dict(tp.source.data)
@@ -214,7 +207,7 @@ def test_TaskProgress(c, s, a, b):
     assert d["name"] == ["slowinc"]
 
     futures2 = c.map(dec, range(5))
-    yield wait(futures2)
+    await wait(futures2)
 
     tp.update()
     d = dict(tp.source.data)
@@ -224,35 +217,35 @@ def test_TaskProgress(c, s, a, b):
     del futures, futures2
 
     while s.tasks:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     tp.update()
     assert not tp.source.data["all"]
 
 
 @gen_cluster(client=True)
-def test_TaskProgress_empty(c, s, a, b):
+async def test_TaskProgress_empty(c, s, a, b):
     tp = TaskProgress(s)
     tp.update()
 
     futures = [c.submit(inc, i, key="f-" + "a" * i) for i in range(20)]
-    yield wait(futures)
+    await wait(futures)
     tp.update()
 
     del futures
     while s.tasks:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
     tp.update()
 
     assert not any(len(v) for v in tp.source.data.values())
 
 
 @gen_cluster(client=True)
-def test_CurrentLoad(c, s, a, b):
+async def test_CurrentLoad(c, s, a, b):
     cl = CurrentLoad(s)
 
     futures = c.map(slowinc, range(10), delay=0.001)
-    yield wait(futures)
+    await wait(futures)
 
     cl.update()
     d = dict(cl.source.data)
@@ -264,34 +257,34 @@ def test_CurrentLoad(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_ProcessingHistogram(c, s, a, b):
+async def test_ProcessingHistogram(c, s, a, b):
     ph = ProcessingHistogram(s)
     ph.update()
     assert (ph.source.data["top"] != 0).sum() == 1
 
     futures = c.map(slowinc, range(10), delay=0.050)
     while not s.tasks:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     ph.update()
     assert ph.source.data["right"][-1] > 2
 
 
 @gen_cluster(client=True)
-def test_NBytesHistogram(c, s, a, b):
+async def test_NBytesHistogram(c, s, a, b):
     nh = NBytesHistogram(s)
     nh.update()
     assert (nh.source.data["top"] != 0).sum() == 1
 
     futures = c.map(inc, range(10))
-    yield wait(futures)
+    await wait(futures)
 
     nh.update()
     assert nh.source.data["right"][-1] > 5 * 20
 
 
 @gen_cluster(client=True)
-def test_WorkerTable(c, s, a, b):
+async def test_WorkerTable(c, s, a, b):
     wt = WorkerTable(s)
     wt.update()
     assert all(wt.source.data.values())
@@ -310,7 +303,7 @@ def test_WorkerTable(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_WorkerTable_custom_metrics(c, s, a, b):
+async def test_WorkerTable_custom_metrics(c, s, a, b):
     def metric_port(worker):
         return worker.port
 
@@ -323,7 +316,7 @@ def test_WorkerTable_custom_metrics(c, s, a, b):
         for name, func in metrics.items():
             w.metrics[name] = func
 
-    yield [a.heartbeat(), b.heartbeat()]
+    await asyncio.gather(a.heartbeat(), b.heartbeat())
 
     for w in [a, b]:
         assert s.workers[w.address].metrics["metric_port"] == w.port
@@ -344,13 +337,13 @@ def test_WorkerTable_custom_metrics(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_WorkerTable_different_metrics(c, s, a, b):
+async def test_WorkerTable_different_metrics(c, s, a, b):
     def metric_port(worker):
         return worker.port
 
     a.metrics["metric_a"] = metric_port
     b.metrics["metric_b"] = metric_port
-    yield [a.heartbeat(), b.heartbeat()]
+    await asyncio.gather(a.heartbeat(), b.heartbeat())
 
     assert s.workers[a.address].metrics["metric_a"] == a.port
     assert s.workers[b.address].metrics["metric_b"] == b.port
@@ -369,12 +362,12 @@ def test_WorkerTable_different_metrics(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_WorkerTable_metrics_with_different_metric_2(c, s, a, b):
+async def test_WorkerTable_metrics_with_different_metric_2(c, s, a, b):
     def metric_port(worker):
         return worker.port
 
     a.metrics["metric_a"] = metric_port
-    yield [a.heartbeat(), b.heartbeat()]
+    await asyncio.gather(a.heartbeat(), b.heartbeat())
 
     wt = WorkerTable(s)
     wt.update()
@@ -388,13 +381,13 @@ def test_WorkerTable_metrics_with_different_metric_2(c, s, a, b):
 
 
 @gen_cluster(client=True, worker_kwargs={"metrics": {"my_port": lambda w: w.port}})
-def test_WorkerTable_add_and_remove_metrics(c, s, a, b):
+async def test_WorkerTable_add_and_remove_metrics(c, s, a, b):
     def metric_port(worker):
         return worker.port
 
     a.metrics["metric_a"] = metric_port
     b.metrics["metric_b"] = metric_port
-    yield [a.heartbeat(), b.heartbeat()]
+    await asyncio.gather(a.heartbeat(), b.heartbeat())
 
     assert s.workers[a.address].metrics["metric_a"] == a.port
     assert s.workers[b.address].metrics["metric_b"] == b.port
@@ -406,14 +399,14 @@ def test_WorkerTable_add_and_remove_metrics(c, s, a, b):
 
     # Remove 'metric_b' from worker b
     del b.metrics["metric_b"]
-    yield [a.heartbeat(), b.heartbeat()]
+    await asyncio.gather(a.heartbeat(), b.heartbeat())
 
     wt = WorkerTable(s)
     wt.update()
     assert "metric_a" in wt.source.data
 
     del a.metrics["metric_a"]
-    yield [a.heartbeat(), b.heartbeat()]
+    await asyncio.gather(a.heartbeat(), b.heartbeat())
 
     wt = WorkerTable(s)
     wt.update()
@@ -421,14 +414,14 @@ def test_WorkerTable_add_and_remove_metrics(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_WorkerTable_custom_metric_overlap_with_core_metric(c, s, a, b):
+async def test_WorkerTable_custom_metric_overlap_with_core_metric(c, s, a, b):
     def metric(worker):
         return -999
 
     a.metrics["executing"] = metric
     a.metrics["cpu"] = metric
     a.metrics["metric"] = metric
-    yield [a.heartbeat(), b.heartbeat()]
+    await asyncio.gather(a.heartbeat(), b.heartbeat())
 
     assert s.workers[a.address].metrics["executing"] != -999
     assert s.workers[a.address].metrics["cpu"] != -999
@@ -436,11 +429,11 @@ def test_WorkerTable_custom_metric_overlap_with_core_metric(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_TaskGraph(c, s, a, b):
+async def test_TaskGraph(c, s, a, b):
     gp = TaskGraph(s)
     futures = c.map(inc, range(5))
     total = c.submit(sum, futures)
-    yield total
+    await total
 
     gp.update()
     assert set(map(len, gp.node_source.data.values())) == {6}
@@ -452,22 +445,22 @@ def test_TaskGraph(c, s, a, b):
     x = da.random.random((20, 20), chunks=(10, 10)).persist()
     y = (x + x.T) - x.mean(axis=0)
     y = y.persist()
-    yield wait(y)
+    await wait(y)
 
     gp.update()
     gp.update()
 
-    yield c.compute((x + y).sum())
+    await c.compute((x + y).sum())
 
     gp.update()
 
     future = c.submit(inc, 10)
     future2 = c.submit(inc, future)
-    yield wait(future2)
+    await wait(future2)
     key = future.key
     del future, future2
     while key in s.tasks:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     assert "memory" in gp.node_source.data["state"]
 
@@ -478,46 +471,69 @@ def test_TaskGraph(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_TaskGraph_clear(c, s, a, b):
+async def test_TaskGraph_clear(c, s, a, b):
     gp = TaskGraph(s)
     futures = c.map(inc, range(5))
     total = c.submit(sum, futures)
-    yield total
+    await total
 
     gp.update()
 
     del total, futures
 
     while s.tasks:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     gp.update()
     gp.update()
 
     start = time()
     while any(gp.node_source.data.values()) or any(gp.edge_source.data.values()):
-        yield gen.sleep(0.1)
+        await asyncio.sleep(0.1)
         gp.update()
         assert time() < start + 5
 
 
+@gen_cluster(
+    client=True, config={"distributed.dashboard.graph-max-items": 2,},
+)
+async def test_TaskGraph_limit(c, s, a, b):
+    gp = TaskGraph(s)
+
+    def func(x):
+        return x
+
+    f1 = c.submit(func, 1)
+    await wait(f1)
+    gp.update()
+    assert len(gp.node_source.data["x"]) == 1
+    f2 = c.submit(func, 2)
+    await wait(f2)
+    gp.update()
+    assert len(gp.node_source.data["x"]) == 2
+    f3 = c.submit(func, 3)
+    await wait(f3)
+    gp.update()
+    assert len(gp.node_source.data["x"]) == 2
+
+
 @gen_cluster(client=True, timeout=30)
-def test_TaskGraph_complex(c, s, a, b):
+async def test_TaskGraph_complex(c, s, a, b):
     da = pytest.importorskip("dask.array")
     gp = TaskGraph(s)
     x = da.random.random((2000, 2000), chunks=(1000, 1000))
     y = ((x + x.T) - x.mean(axis=0)).persist()
-    yield wait(y)
+    await wait(y)
     gp.update()
     assert len(gp.layout.index) == len(gp.node_source.data["x"])
     assert len(gp.layout.index) == len(s.tasks)
     z = (x - y).sum().persist()
-    yield wait(z)
+    await wait(z)
     gp.update()
     assert len(gp.layout.index) == len(gp.node_source.data["x"])
     assert len(gp.layout.index) == len(s.tasks)
     del z
-    yield gen.sleep(0.2)
+    await asyncio.sleep(0.2)
     gp.update()
     assert len(gp.layout.index) == sum(
         v == "True" for v in gp.node_source.data["visible"]
@@ -533,10 +549,10 @@ def test_TaskGraph_complex(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_TaskGraph_order(c, s, a, b):
+async def test_TaskGraph_order(c, s, a, b):
     x = c.submit(inc, 1)
     y = c.submit(div, 1, 0)
-    yield wait(y)
+    await wait(y)
 
     gp = TaskGraph(s)
     gp.update()
@@ -551,35 +567,33 @@ def test_TaskGraph_order(c, s, a, b):
         "distributed.worker.profile.cycle": "50ms",
     },
 )
-def test_profile_server(c, s, a, b):
+async def test_profile_server(c, s, a, b):
     ptp = ProfileServer(s)
     start = time()
-    yield gen.sleep(0.100)
+    await asyncio.sleep(0.100)
     while len(ptp.ts_source.data["time"]) < 2:
-        yield gen.sleep(0.100)
+        await asyncio.sleep(0.100)
         ptp.trigger_update()
         assert time() < start + 2
 
 
 @gen_cluster(
-    client=True, scheduler_kwargs={"services": {("dashboard", 0): BokehScheduler}}
+    client=True, scheduler_kwargs={"dashboard": True},
 )
-def test_root_redirect(c, s, a, b):
+async def test_root_redirect(c, s, a, b):
     http_client = AsyncHTTPClient()
-    response = yield http_client.fetch(
-        "http://localhost:%d/" % s.services["dashboard"].port
-    )
+    response = await http_client.fetch("http://localhost:%d/" % s.http_server.port)
     assert response.code == 200
     assert "/status" in response.effective_url
 
 
 @gen_cluster(
     client=True,
-    scheduler_kwargs={"services": {("dashboard", 0): BokehScheduler}},
-    worker_kwargs={"services": {"dashboard": BokehWorker}},
+    scheduler_kwargs={"dashboard": True},
+    worker_kwargs={"dashboard": True},
     timeout=180,
 )
-def test_proxy_to_workers(c, s, a, b):
+async def test_proxy_to_workers(c, s, a, b):
     try:
         import jupyter_server_proxy  # noqa: F401
 
@@ -587,15 +601,15 @@ def test_proxy_to_workers(c, s, a, b):
     except ImportError:
         proxy_exists = False
 
-    dashboard_port = s.services["dashboard"].port
+    dashboard_port = s.http_server.port
     http_client = AsyncHTTPClient()
-    response = yield http_client.fetch("http://localhost:%d/" % dashboard_port)
+    response = await http_client.fetch("http://localhost:%d/" % dashboard_port)
     assert response.code == 200
     assert "/status" in response.effective_url
 
     for w in [a, b]:
         host = w.ip
-        port = w.service_ports["dashboard"]
+        port = w.http_server.port
         proxy_url = "http://localhost:%d/proxy/%s/%s/status" % (
             dashboard_port,
             port,
@@ -603,8 +617,8 @@ def test_proxy_to_workers(c, s, a, b):
         )
         direct_url = "http://localhost:%s/status" % port
         http_client = AsyncHTTPClient()
-        response_proxy = yield http_client.fetch(proxy_url)
-        response_direct = yield http_client.fetch(direct_url)
+        response_proxy = await http_client.fetch(proxy_url)
+        response_direct = await http_client.fetch(direct_url)
 
         assert response_proxy.code == 200
         if proxy_exists:
@@ -617,7 +631,7 @@ def test_proxy_to_workers(c, s, a, b):
 
 @gen_cluster(
     client=True,
-    scheduler_kwargs={"services": {("dashboard", 0): BokehScheduler}},
+    scheduler_kwargs={"dashboard": True},
     config={
         "distributed.scheduler.dashboard.tasks.task-stream-length": 10,
         "distributed.scheduler.dashboard.status.task-stream-length": 10,
@@ -645,16 +659,15 @@ async def test_lots_of_tasks(c, s, a, b):
 
 @gen_cluster(
     client=True,
-    scheduler_kwargs={"services": {("dashboard", 0): BokehScheduler}},
+    scheduler_kwargs={"dashboard": True},
     config={
         "distributed.scheduler.dashboard.tls.key": get_cert("tls-key.pem"),
         "distributed.scheduler.dashboard.tls.cert": get_cert("tls-cert.pem"),
         "distributed.scheduler.dashboard.tls.ca-file": get_cert("tls-ca-cert.pem"),
     },
 )
-def test_https_support(c, s, a, b):
-    assert isinstance(s.services["dashboard"], BokehScheduler)
-    port = s.services["dashboard"].port
+async def test_https_support(c, s, a, b):
+    port = s.http_server.port
 
     assert (
         format_dashboard_link("localhost", port) == "https://localhost:%d/status" % port
@@ -664,7 +677,7 @@ def test_https_support(c, s, a, b):
     ctx.load_verify_locations(get_cert("tls-ca-cert.pem"))
 
     http_client = AsyncHTTPClient()
-    response = yield http_client.fetch(
+    response = await http_client.fetch(
         "https://localhost:%d/individual-plots.json" % port, ssl_options=ctx
     )
     response = json.loads(response.body.decode())
@@ -681,15 +694,13 @@ def test_https_support(c, s, a, b):
         req = HTTPRequest(
             url="https://localhost:%d/%s" % (port, suffix), ssl_options=ctx
         )
-        response = yield http_client.fetch(req)
+        response = await http_client.fetch(req)
         assert response.code < 300
         body = response.body.decode()
         assert not re.search("href=./", body)  # no absolute links
 
 
-@gen_cluster(
-    client=True, scheduler_kwargs={"services": {("dashboard", 0): BokehScheduler}}
-)
+@gen_cluster(client=True, scheduler_kwargs={"dashboard": True})
 async def test_memory_by_key(c, s, a, b):
     mbk = MemoryByKey(s)
 
@@ -702,3 +713,21 @@ async def test_memory_by_key(c, s, a, b):
     mbk.update()
     assert mbk.source.data["name"] == ["add", "inc"]
     assert mbk.source.data["nbytes"] == [x.nbytes, sys.getsizeof(1)]
+
+
+@gen_cluster(scheduler_kwargs={"http_prefix": "foo-bar", "dashboard": True})
+async def test_prefix_bokeh(s, a, b):
+    prefix = "foo-bar"
+    http_client = AsyncHTTPClient()
+    response = await http_client.fetch(
+        f"http://localhost:{s.http_server.port}/{prefix}/status"
+    )
+    assert response.code == 200
+    assert (
+        f'<script type="text/javascript" src="/{prefix}/static/'
+        in response.body.decode()
+    )
+
+    bokeh_app = s.http_application.applications[0]
+    assert isinstance(bokeh_app, BokehTornado)
+    assert bokeh_app.prefix == f"/{prefix}"
