@@ -1,20 +1,18 @@
+import asyncio
 from collections import defaultdict, deque
-import datetime
 import logging
 import threading
 import weakref
 
-import tornado.locks
-from tornado import gen
-
 from .core import CommClosedError
-from .utils import sync, TimeoutError
+from .metrics import time
+from .utils import sync, TimeoutError, ignoring
 from .protocol.serialize import to_serialize
 
 logger = logging.getLogger(__name__)
 
 
-class PubSubSchedulerExtension(object):
+class PubSubSchedulerExtension:
     """ Extend Dask's scheduler with routes to handle PubSub machinery """
 
     def __init__(self, scheduler):
@@ -117,7 +115,7 @@ class PubSubSchedulerExtension(object):
                 )
 
 
-class PubSubWorkerExtension(object):
+class PubSubWorkerExtension:
     """ Extend Dask's Worker with routes to handle PubSub machinery """
 
     def __init__(self, worker):
@@ -148,9 +146,9 @@ class PubSubWorkerExtension(object):
     def publish_scheduler(self, name=None, publish=None):
         self.publish_to_scheduler[name] = publish
 
-    def handle_message(self, name=None, msg=None):
+    async def handle_message(self, name=None, msg=None):
         for sub in self.subscribers.get(name, []):
-            sub._put(msg)
+            await sub._put(msg)
 
     def trigger_cleanup(self):
         self.worker.loop.add_callback(self.cleanup)
@@ -170,7 +168,7 @@ class PubSubWorkerExtension(object):
                 del self.publish_to_scheduler[name]
 
 
-class PubSubClientExtension(object):
+class PubSubClientExtension:
     """ Extend Dask's Client with handlers to handle PubSub machinery """
 
     def __init__(self, client):
@@ -180,9 +178,9 @@ class PubSubClientExtension(object):
         self.subscribers = defaultdict(weakref.WeakSet)
         self.client.extensions["pubsub"] = self  # TODO: circular reference
 
-    def handle_message(self, name=None, msg=None):
+    async def handle_message(self, name=None, msg=None):
         for sub in self.subscribers[name]:
-            sub._put(msg)
+            await sub._put(msg)
 
         if not self.subscribers[name]:
             self.client.scheduler_comm.send(
@@ -199,7 +197,7 @@ class PubSubClientExtension(object):
                 self.client.scheduler_comm.send(msg)
 
 
-class Pub(object):
+class Pub:
     """ Publish data with Publish-Subscribe pattern
 
     This allows clients and workers to directly communicate data between each
@@ -240,7 +238,13 @@ class Pub(object):
     Parameters
     ----------
     name: object (msgpack serializable)
-        The name of the group of Pubs and Subs on which to participate
+        The name of the group of Pubs and Subs on which to participate.
+    worker: Worker (optional)
+        The worker to be used for publishing data. Defaults to the value of
+        ```get_worker()```. If given, ``client`` must be ``None``.
+    client: Client (optional)
+        Client used for communication with the scheduler. Defaults to
+        the value of ``get_client()``. If given, ``worker`` must be ``None``.
 
     Examples
     --------
@@ -349,7 +353,7 @@ class Pub(object):
     __str__ = __repr__
 
 
-class Sub(object):
+class Sub:
     """ Subscribe to a Publish/Subscribe topic
 
     See Also
@@ -374,7 +378,6 @@ class Sub(object):
             self.loop = self.client.loop
         self.name = name
         self.buffer = deque()
-        self.condition = tornado.locks.Condition()
 
         if self.worker:
             pubsub = self.worker.extensions["pubsub"]
@@ -392,21 +395,33 @@ class Sub(object):
 
         weakref.finalize(self, pubsub.trigger_cleanup)
 
+    @property
+    def condition(self):
+        try:
+            return self._condition
+        except AttributeError:
+            self._condition = asyncio.Condition()
+            return self._condition
+
     async def _get(self, timeout=None):
-        if timeout is not None:
-            timeout = datetime.timedelta(seconds=timeout)
-        start = datetime.datetime.now()
+        start = time()
         while not self.buffer:
             if timeout is not None:
-                timeout2 = timeout - (datetime.datetime.now() - start)
-                if timeout2.total_seconds() < 0:
+                timeout2 = timeout - (time() - start)
+                if timeout2 < 0:
                     raise TimeoutError()
             else:
                 timeout2 = None
+
+            async def _():
+                await self.condition.acquire()
+                await self.condition.wait()
+
             try:
-                await self.condition.wait(timeout=timeout2)
-            except gen.TimeoutError:
-                raise TimeoutError("Timed out waiting on Sub")
+                await asyncio.wait_for(_(), timeout2)
+            finally:
+                with ignoring(RuntimeError):  # Python 3.6 fails here sometimes
+                    self.condition.release()
 
         return self.buffer.popleft()
 
@@ -431,9 +446,10 @@ class Sub(object):
     def __aiter__(self):
         return self
 
-    def _put(self, msg):
+    async def _put(self, msg):
         self.buffer.append(msg)
-        self.condition.notify()
+        async with self.condition:
+            self.condition.notify()
 
     def __repr__(self):
         return "<Sub: {}>".format(self.name)
