@@ -1,10 +1,10 @@
+import asyncio
 import os
 import random
 from time import sleep
 
 import pytest
-from toolz import partition_all, first
-from tornado import gen
+from tlz import partition_all, first
 
 from dask import delayed
 from distributed import Client, Nanny, wait
@@ -35,30 +35,30 @@ def test_submit_after_failed_worker_sync(loop):
 
 
 @gen_cluster(client=True, timeout=60, active_rpc_timeout=10)
-def test_submit_after_failed_worker_async(c, s, a, b):
-    n = yield Nanny(s.address, nthreads=2, loop=s.loop)
+async def test_submit_after_failed_worker_async(c, s, a, b):
+    n = await Nanny(s.address, nthreads=2, loop=s.loop)
     while len(s.workers) < 3:
-        yield gen.sleep(0.1)
+        await asyncio.sleep(0.1)
 
     L = c.map(inc, range(10))
-    yield wait(L)
+    await wait(L)
 
     s.loop.add_callback(n.kill)
     total = c.submit(sum, L)
-    result = yield total
+    result = await total
     assert result == sum(map(inc, range(10)))
 
-    yield n.close()
+    await n.close()
 
 
 @gen_cluster(client=True, timeout=60)
-def test_submit_after_failed_worker(c, s, a, b):
+async def test_submit_after_failed_worker(c, s, a, b):
     L = c.map(inc, range(10))
-    yield wait(L)
-    yield a.close()
+    await wait(L)
+    await a.close()
 
     total = c.submit(sum, L)
-    result = yield total
+    result = await total
     assert result == sum(map(inc, range(10)))
 
 
@@ -78,73 +78,73 @@ def test_gather_after_failed_worker(loop):
     nthreads=[("127.0.0.1", 1)] * 4,
     config={"distributed.comm.timeouts.connect": "1s"},
 )
-def test_gather_then_submit_after_failed_workers(c, s, w, x, y, z):
+async def test_gather_then_submit_after_failed_workers(c, s, w, x, y, z):
     L = c.map(inc, range(20))
-    yield wait(L)
+    await wait(L)
 
     w.process.process._process.terminate()
     total = c.submit(sum, L)
 
     for i in range(3):
-        yield wait(total)
+        await wait(total)
         addr = first(s.tasks[total.key].who_has).address
         for worker in [x, y, z]:
             if worker.worker_address == addr:
                 worker.process.process._process.terminate()
                 break
 
-        result = yield c.gather([total])
+        result = await c.gather([total])
         assert result == [sum(map(inc, range(20)))]
 
 
 @gen_cluster(Worker=Nanny, timeout=60, client=True)
-def test_failed_worker_without_warning(c, s, a, b):
+async def test_failed_worker_without_warning(c, s, a, b):
     L = c.map(inc, range(10))
-    yield wait(L)
+    await wait(L)
 
     original_pid = a.pid
     with ignoring(CommClosedError):
-        yield c._run(os._exit, 1, workers=[a.worker_address])
+        await c._run(os._exit, 1, workers=[a.worker_address])
     start = time()
     while a.pid == original_pid:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() - start < 10
 
-    yield gen.sleep(0.5)
+    await asyncio.sleep(0.5)
 
     start = time()
     while len(s.nthreads) < 2:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() - start < 10
 
-    yield wait(L)
+    await wait(L)
 
     L2 = c.map(inc, range(10, 20))
-    yield wait(L2)
+    await wait(L2)
     assert all(len(keys) > 0 for keys in s.has_what.values())
     nthreads2 = dict(s.nthreads)
 
-    yield c.restart()
+    await c.restart()
 
     L = c.map(inc, range(10))
-    yield wait(L)
+    await wait(L)
     assert all(len(keys) > 0 for keys in s.has_what.values())
 
     assert not (set(nthreads2) & set(s.nthreads))  # no overlap
 
 
 @gen_cluster(Worker=Nanny, client=True, timeout=60)
-def test_restart(c, s, a, b):
+async def test_restart(c, s, a, b):
     assert s.nthreads == {a.worker_address: 1, b.worker_address: 2}
 
     x = c.submit(inc, 1)
     y = c.submit(inc, x)
     z = c.submit(div, 1, 0)
-    yield y
+    await y
 
     assert set(s.who_has) == {x.key, y.key}
 
-    f = yield c.restart()
+    f = await c.restart()
     assert f is c
 
     assert len(s.workers) == 2
@@ -162,12 +162,12 @@ def test_restart(c, s, a, b):
 
 
 @gen_cluster(Worker=Nanny, client=True, timeout=60)
-def test_restart_cleared(c, s, a, b):
+async def test_restart_cleared(c, s, a, b):
     x = 2 * delayed(1) + 1
     f = c.compute(x)
-    yield wait([f])
+    await wait([f])
 
-    yield c.restart()
+    await c.restart()
 
     for coll in [s.tasks, s.unrunnable]:
         assert not coll
@@ -204,18 +204,18 @@ def test_restart_sync(loop):
 
 
 @gen_cluster(Worker=Nanny, client=True, timeout=60)
-def test_restart_fast(c, s, a, b):
+async def test_restart_fast(c, s, a, b):
     L = c.map(sleep, range(10))
 
     start = time()
-    yield c.restart()
+    await c.restart()
     assert time() - start < 10
     assert len(s.nthreads) == 2
 
     assert all(x.status == "cancelled" for x in L)
 
     x = c.submit(inc, 1)
-    result = yield x
+    result = await x
     assert result == 2
 
 
@@ -247,51 +247,51 @@ def test_restart_fast_sync(loop):
 
 
 @gen_cluster(Worker=Nanny, client=True, timeout=60)
-def test_fast_kill(c, s, a, b):
+async def test_fast_kill(c, s, a, b):
     L = c.map(sleep, range(10))
 
     start = time()
-    yield c.restart()
+    await c.restart()
     assert time() - start < 10
 
     assert all(x.status == "cancelled" for x in L)
 
     x = c.submit(inc, 1)
-    result = yield x
+    result = await x
     assert result == 2
 
 
 @gen_cluster(Worker=Nanny, timeout=60)
-def test_multiple_clients_restart(s, a, b):
-    c1 = yield Client(s.address, asynchronous=True)
-    c2 = yield Client(s.address, asynchronous=True)
+async def test_multiple_clients_restart(s, a, b):
+    c1 = await Client(s.address, asynchronous=True)
+    c2 = await Client(s.address, asynchronous=True)
 
     x = c1.submit(inc, 1)
     y = c2.submit(inc, 2)
-    xx = yield x
-    yy = yield y
+    xx = await x
+    yy = await y
     assert xx == 2
     assert yy == 3
 
-    yield c1.restart()
+    await c1.restart()
 
     assert x.cancelled()
     start = time()
     while not y.cancelled():
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() < start + 5
 
-    yield c1.close()
-    yield c2.close()
+    await c1.close()
+    await c2.close()
 
 
 @gen_cluster(Worker=Nanny, timeout=60)
-def test_restart_scheduler(s, a, b):
+async def test_restart_scheduler(s, a, b):
     import gc
 
     gc.collect()
     addrs = (a.worker_address, b.worker_address)
-    yield s.restart()
+    await s.restart()
     assert len(s.nthreads) == 2
     addrs2 = (a.worker_address, b.worker_address)
 
@@ -299,26 +299,26 @@ def test_restart_scheduler(s, a, b):
 
 
 @gen_cluster(Worker=Nanny, client=True, timeout=60)
-def test_forgotten_futures_dont_clean_up_new_futures(c, s, a, b):
+async def test_forgotten_futures_dont_clean_up_new_futures(c, s, a, b):
     x = c.submit(inc, 1)
-    yield c.restart()
+    await c.restart()
     y = c.submit(inc, 1)
     del x
     import gc
 
     gc.collect()
-    yield gen.sleep(0.1)
-    yield y
+    await asyncio.sleep(0.1)
+    await y
 
 
 @gen_cluster(client=True, timeout=60, active_rpc_timeout=10)
-def test_broken_worker_during_computation(c, s, a, b):
+async def test_broken_worker_during_computation(c, s, a, b):
     s.allowed_failures = 100
-    n = yield Nanny(s.address, nthreads=2, loop=s.loop)
+    n = await Nanny(s.address, nthreads=2, loop=s.loop)
 
     start = time()
     while len(s.nthreads) < 3:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() < start + 5
 
     N = 256
@@ -333,37 +333,37 @@ def test_broken_worker_during_computation(c, s, a, b):
             key=["add-%d-%d" % (i, j) for j in range(len(L) // 2)]
         )
 
-    yield gen.sleep(random.random() / 20)
+    await asyncio.sleep(random.random() / 20)
     with ignoring(CommClosedError):  # comm will be closed abrupty
-        yield c._run(os._exit, 1, workers=[n.worker_address])
+        await c._run(os._exit, 1, workers=[n.worker_address])
 
-    yield gen.sleep(random.random() / 20)
+    await asyncio.sleep(random.random() / 20)
     while len(s.workers) < 3:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     with ignoring(
         CommClosedError, EnvironmentError
     ):  # perhaps new worker can't be contacted yet
-        yield c._run(os._exit, 1, workers=[n.worker_address])
+        await c._run(os._exit, 1, workers=[n.worker_address])
 
-    [result] = yield c.gather(L)
+    [result] = await c.gather(L)
     assert isinstance(result, int)
     assert result == expected_result
 
-    yield n.close()
+    await n.close()
 
 
 @gen_cluster(client=True, Worker=Nanny, timeout=60)
-def test_restart_during_computation(c, s, a, b):
+async def test_restart_during_computation(c, s, a, b):
     xs = [delayed(slowinc)(i, delay=0.01) for i in range(50)]
     ys = [delayed(slowinc)(i, delay=0.01) for i in xs]
     zs = [delayed(slowadd)(x, y, delay=0.01) for x, y in zip(xs, ys)]
     total = delayed(sum)(zs)
     result = c.compute(total)
 
-    yield gen.sleep(0.5)
+    await asyncio.sleep(0.5)
     assert s.rprocessing
-    yield c.restart()
+    await c.restart()
     assert not s.rprocessing
 
     assert len(s.nthreads) == 2
@@ -371,59 +371,59 @@ def test_restart_during_computation(c, s, a, b):
 
 
 @gen_cluster(client=True, timeout=60)
-def test_worker_who_has_clears_after_failed_connection(c, s, a, b):
-    n = yield Nanny(s.address, nthreads=2, loop=s.loop)
+async def test_worker_who_has_clears_after_failed_connection(c, s, a, b):
+    n = await Nanny(s.address, nthreads=2, loop=s.loop)
 
     start = time()
     while len(s.nthreads) < 3:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() < start + 5
 
     futures = c.map(slowinc, range(20), delay=0.01, key=["f%d" % i for i in range(20)])
-    yield wait(futures)
+    await wait(futures)
 
-    result = yield c.submit(sum, futures, workers=a.address)
+    result = await c.submit(sum, futures, workers=a.address)
     for dep in set(a.dep_state) - set(a.task_state):
         a.release_dep(dep, report=True)
 
     n_worker_address = n.worker_address
     with ignoring(CommClosedError):
-        yield c._run(os._exit, 1, workers=[n_worker_address])
+        await c._run(os._exit, 1, workers=[n_worker_address])
 
     while len(s.workers) > 2:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     total = c.submit(sum, futures, workers=a.address)
-    yield total
+    await total
 
     assert not a.has_what.get(n_worker_address)
     assert not any(n_worker_address in s for s in a.who_has.values())
 
-    yield n.close()
+    await n.close()
 
 
 @pytest.mark.slow
 @gen_cluster(client=True, timeout=60, Worker=Nanny, nthreads=[("127.0.0.1", 1)])
-def test_restart_timeout_on_long_running_task(c, s, a):
+async def test_restart_timeout_on_long_running_task(c, s, a):
     with captured_logger("distributed.scheduler") as sio:
         future = c.submit(sleep, 3600)
-        yield gen.sleep(0.1)
-        yield c.restart(timeout=20)
+        await asyncio.sleep(0.1)
+        await c.restart(timeout=20)
 
     text = sio.getvalue()
     assert "timeout" not in text.lower()
 
 
 @gen_cluster(client=True, scheduler_kwargs={"worker_ttl": "500ms"})
-def test_worker_time_to_live(c, s, a, b):
+async def test_worker_time_to_live(c, s, a, b):
     assert set(s.workers) == {a.address, b.address}
     a.periodic_callbacks["heartbeat"].stop()
-    yield gen.sleep(0.010)
+    await asyncio.sleep(0.010)
     assert set(s.workers) == {a.address, b.address}
 
     start = time()
     while set(s.workers) == {a.address, b.address}:
-        yield gen.sleep(0.050)
+        await asyncio.sleep(0.050)
         assert time() < start + 2
 
     set(s.workers) == {b.address}
